// IceCreamOrder.cpp : Implementation of CIceCreamOrder
#include "stdafx.h"
#include "Icecream.h"
#include "IceCreamOrder.h"

/////////////////////////////////////////////////////////////////////////////
// CIceCreamOrder



//////////////////////////////////////////////////////////////////////////////
//Function : GetFlavorsWithPrices (Example for Multidimensional SAFEARRAY)
//Parameters: VARIANT (out parameter that contains a SAFEARRAY of VARIANTs
//			   helping us to pass BSTR and float in the same array)
//Return Type : HRESULT
//////////////////////////////////////////////////////////////////////////////


STDMETHODIMP CIceCreamOrder::GetFlavorsWithPrices(VARIANT *pVariant)
{
	// TODO: Add your implementation code here
	
	
	
	//Initialize the bounds for the array
	//Ours is a 2 dimensional array
	SAFEARRAYBOUND safeBound[2]; 
	
	//Set up the bounds for the first index
	//That's the number of flavors that we have
	safeBound[0].cElements = m_vecIcecreamFlavors.size();    
	safeBound[0].lLbound = 0;

	//Set up the bounds for the second index
	safeBound[1].cElements = m_vecIcecreamPrices.size();
	safeBound[1].lLbound = 0 ;

	
	///Initialize the VARIANT
	VariantInit(pVariant);
	//The array type is VARIANT
	//Storage will accomodate a BSTR and a float
	pVariant->vt = VT_VARIANT | VT_ARRAY; 
	pVariant->parray = SafeArrayCreate(VT_VARIANT,2,safeBound);
	
	
	//Initialize the vector iterators
	std::vector<CComBSTR>::iterator iterFlavor;
	std::vector<float>::iterator iterPrices;


	
	//Used for indicating indexes in the Multidimensional array
	long lDimension[2];
	int iFlavorIndex = 0;

	
	//Start iteration
	iterPrices = m_vecIcecreamPrices.begin();
	iterFlavor = m_vecIcecreamFlavors.begin(); 


	//Iterate thru the list of flavors and prices
	while(iterFlavor != m_vecIcecreamFlavors.end()) 
	{

		
		//Put the Element at (0,0), (0,1)  , (0,2) ,.............(0,n)
		lDimension[1] = iFlavorIndex;
		lDimension[0] = 0;
		CComVariant variantFlavor(SysAllocString((*iterFlavor).m_str));
		SafeArrayPutElement(pVariant->parray,lDimension,&variantFlavor);

	
		
		//Put the Element at (1,0), (1,1)  , (1,2) ,.............(1,n)
		lDimension[1] = iFlavorIndex;
		lDimension[0] = 1;
		CComVariant variantPrices(*iterPrices);
		SafeArrayPutElement(pVariant->parray,lDimension,&variantPrices);
	
		
		iFlavorIndex++;
		iterPrices++;
		iterFlavor++;
		

	}
	
	return S_OK;

}








//////////////////////////////////////////////////////////////////////////////
//Function : GetFlavors (Example for single dimensional SAFEARRAY)
//Parameters: VARIANT (out parameter that contains a SAFEARRAY of BSTRs)
//Return Type : HRESULT
//////////////////////////////////////////////////////////////////////////////


STDMETHODIMP CIceCreamOrder::GetFlavors(VARIANT *pVariant)
{
	// TODO: Add your implementation code here
	


	//Initialize the VARIANT (Type is SAFEARRAY of BSTRs)
	VariantInit(pVariant);
	pVariant->vt = VT_ARRAY | VT_BSTR;


	
	SAFEARRAY* pSafeArray = NULL;
	//Safe Array size is  the number of flavors that we have
	SAFEARRAYBOUND safeBounds = { m_vecIcecreamFlavors.size() , 0};


	//Create the Safe Array passing it the bounds
	pSafeArray = SafeArrayCreate(VT_BSTR, 1, &safeBounds);


	//Get a pointer to the array data
	//(This actually increments the array's lock count)
	BSTR* bstrFlavorArray = NULL;
	SafeArrayAccessData(pSafeArray, (void**)&bstrFlavorArray);

	//Vector Iterator for navigating
	std::vector<CComBSTR>::iterator iterVec;

	int iFlavorIndex = 0;

	// Iterate through the array
	for (iterVec = m_vecIcecreamFlavors.begin();
		iterVec != m_vecIcecreamFlavors.end(); 
		iterVec++)
	{
		
		//Assign each flavor that is available to the array
		bstrFlavorArray[iFlavorIndex] = SysAllocString((*iterVec).m_str);
		iFlavorIndex++;

	}

	//We are done wth populating the array
	//Decrement the array's lock count
	SafeArrayUnaccessData(pSafeArray);


	//Assign our VARIANT out param with the array	
	pVariant->parray = pSafeArray;

	return S_OK;

}




 



